@@ -1,6 +1,6 @@
 use std::sync::Arc;
 
-use tracing::debug;
+use tracing::{debug, instrument};
 
 use fluvio_socket::SharedMultiplexerSocket;
 use fluvio_socket::SocketError;
@@ -24,7 +24,10 @@ pub struct MetadataStores {
 
 impl MetadataStores {
     /// start synchronization
+
+    #[instrument(skip(socket))]
     pub async fn start(socket: SharedMultiplexerSocket) -> Result<Self, SocketError> {
+        debug!("starting metadata store");
         let store = Self {
             shutdown: SimpleEvent::shared(),
             spus: StoreContext::new(),
@@ -57,13 +60,13 @@ impl MetadataStores {
     }
 
     /// start watch for spu
+    #[instrument(skip(self))]
     pub async fn start_watch_for_spu(&self) -> Result<(), SocketError> {
         use dataplane::api::RequestMessage;
         use fluvio_sc_schema::objects::WatchRequest;
 
-        debug!("start watch for spu");
-
         let req_msg = RequestMessage::new_request(WatchRequest::Spu(0));
+        debug!("create spu metadata stream");
         let async_response = self.socket.create_stream(req_msg, 10).await?;
 
         MetadataSyncController::<SpuSpec>::start(
@@ -75,6 +78,7 @@ impl MetadataStores {
         Ok(())
     }
 
+    #[instrument(skip(self))]
     pub async fn start_watch_for_partition(&self) -> Result<(), SocketError> {
         use dataplane::api::RequestMessage;
         use fluvio_sc_schema::objects::WatchRequest;
@@ -93,6 +97,7 @@ impl MetadataStores {
         Ok(())
     }
 
+    #[instrument(skip(self))]
     pub async fn start_watch_for_topic(&self) -> Result<(), SocketError> {
         use dataplane::api::RequestMessage;
         use fluvio_sc_schema::objects::WatchRequest;
